/**
 * arxtools - Tools for modififying ARX archives.
 * Copyright (C) 2005  Hagen Möbius
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/

#include <dirent.h>
#include <errno.h>
#include <sys/stat.h>

#include <iostream>
#include <fstream>
#include <sstream>

#include <BufferWriter.h>
#include <DataChannel.h>
#include <DataRepository.h>
#include <ItemFactory.h>

#include "common.h"

Arxx::u4byte g_u4UniqueID = Arxx::g_u4InvalidID;
std::string g_sName = "";
Arxx::u4byte g_u4Type = 0xFFFFFFFF;
Arxx::u4byte g_u4SubType = 0xFFFFFFFF;
Arxx::u4byte g_u4Version = 0;
bool g_bCompress = false;
bool g_bVerbose = false;
bool g_External(false);
Arxx::u4byte g_u4CompressThreshold = 32;
std::multiset< std::pair< Arxx::u4byte, std::string > > g_AddToRelations;

Arxx::Item * pAddItem(Arxx::Archive & Archive, const Arxx::URI & ItemURI);
Arxx::Item * pAddDirectory(Arxx::Archive & Archive, const std::string & sDirectoryName);
Arxx::Item * pAddFile(Arxx::Archive & Archive, const std::string & sFileName);

class LocalFileChannel : public Arxx::DataChannel
{
public:
	LocalFileChannel(void) :
		Arxx::DataChannel("file://")
	{
	}
	
	virtual bool bFetchData(const Arxx::URI & URI, Arxx::Buffer & Buffer, Arxx::FetchStatus & FetchStatus)
	{
		if(URI.sGetScheme() != "file")
		{
			std::cerr << " *** LocalFileChannel::bFetchData called with scheme \"" << URI.sGetScheme() << "\" but \"file\" was expected." << std::endl;
			
			return false;
		}
		if((URI.sGetAuthority() != "") && (URI.sGetAuthority() != "localhost") && (URI.sGetAuthority() != "127.0.0.1"))
		{
			std::cerr << " *** LocalFileChannel::bFetchData called with authority \"" << URI.sGetAuthority() << "\" but \"\", \"localhost\" or \"127.0.0.1\" was expected." << std::endl;
			
			return false;
		}
		
		std::ifstream IStream(URI.sGetPath().c_str());
		
		if(IStream == false)
		{
			std::cerr << " *** LocalFileChannel::bFetchData: Could not open file \"" << URI << "\"." << std::endl;
			
			return false;
		}
		Buffer.vSetLength(0);
		
		Arxx::BufferWriter BufferWriter(Buffer);
		
		FetchStatus = Arxx::TRANSFERING;
		BufferWriter << std::make_pair(static_cast< Arxx::Buffer::size_type >(0xFFFFFFFF), reinterpret_cast< std::istream * >(&IStream));
		IStream.close();
		FetchStatus = Arxx::FETCHED;
		
		return true;
	}
} g_LocalFileChannel;

bool bParameters(std::vector< std::string > & Parameters)
{
	if((Parameters.size() > 0) && (Parameters[0] == "--information"))
	{
		std::cerr << "arxadd as part of " PACKAGE_STRING "\n\n";
		std::cerr << "No warranty, whatsoever. Copyright by Hagen Moebius, 2005\n" << std::endl;
		
		return false;
	}
	if((Parameters.size() > 0) && (Parameters[0] == "--version"))
	{
		std::cerr << PACKAGE_VERSION << std::endl;
		
		return false;
	}
	if((Parameters.size() == 0) || (Parameters[0] == "--help") || (Parameters.size() < 2))
	{
		std::cerr << "Usage: arxadd <arx archive> ((<option>)* <file>)+\n\n";
		std::cerr << "<option> may be any number of:\n";
		std::cerr << "  --uid <id>                     - set the hexadecimal unique id of the next item to <id>\n";
		std::cerr << "  --name <name>                  - set the name of every following item to <name>\n";
		std::cerr << "  --type <type>                  - set the type of every following item to <type>\n";
		std::cerr << "  --subtype <subtype>            - set the sub type of every following item to <subtype>\n";
		std::cerr << "  --version <version>            - set the version of every following item to <version>\n";
		std::cerr << "  --add <uid|path>[::<relation>] - add the new item to <item>'s <relation> (default \"child\")";
		std::cerr << "  --external                     - make following item's data external\n";
		std::cerr << "  --internal                     - internalize the following item's data\n";
		std::cerr << "  --compress                     - compress every following item\n";
		std::cerr << "  --threshold <threshold>        - only compress items of size greater than <threshold>\n";
		std::cerr << "  --verbose                      - toggles the verbose flag\n\n";
		std::cerr << "No warranty, whatsoever. Copyright by Hagen Moebius, 2005\n" << std::endl;
		
		return false;
	}
	
	return true;
}

void vAssignUniqueID(Arxx::Item * pItem)
{
	if(g_u4UniqueID != Arxx::g_u4InvalidID)
	{
		bool bDone(false);
		
		while(bDone == false)
		{
			bDone = true;
			try
			{
				pItem->vSetUniqueID(g_u4UniqueID++);
			}
			catch(Arxx::id_not_unique & Exception)
			{
				bDone = false;
			}
		}
	}
}

Arxx::Item * pAddFile(Arxx::Archive & Archive, const Arxx::URI & FileURI)
{
	Arxx::Item * pItem(Archive.pGetItemFactory()->pNewItem(Archive));
	
	pItem->vSetExternal(FileURI);
	if(g_External == false)
	{
		pItem->bFetch();
		if(pItem->bIsFetched() == true)
		{
			pItem->vInternalize();
			if((g_bCompress == true) && (pItem->u4GetDecompressedLength() >= g_u4CompressThreshold))
			{
				pItem->vCompress();
			}
		}
	}
	vAssignUniqueID(pItem);
	
	return pItem;
}

Arxx::Item * pAddDirectory(Arxx::Archive & Archive, const Arxx::URI & DirectoryURI)
{
	Arxx::Item * pItem(Archive.pGetItemFactory()->pNewItem(Archive));
	DIR * pDirectory(opendir(DirectoryURI.sGetPath().c_str()));
	
	vAssignUniqueID(pItem);
	if(pDirectory != 0)
	{
		while(true)
		{
			dirent * pDirectoryEntry(readdir(pDirectory));
			
			if(pDirectoryEntry == 0)
			{
				break;
			}
			if((std::string(".") == pDirectoryEntry->d_name) || (std::string("..") == pDirectoryEntry->d_name))
			{
				continue;
			}
			
			Arxx::URI ItemURI(DirectoryURI);
			
			ItemURI.vSetPath(ItemURI.sGetPath() + '/' + pDirectoryEntry->d_name);
			
			Arxx::Item * pChild(pAddItem(Archive, ItemURI));
			
			if(pChild != 0)
			{
				pItem->GetStructure().bAdd(pChild->u4GetUniqueID(), "child");
				pChild->GetStructure().bAdd(pItem->u4GetUniqueID(), "parent");
			}
		}
		closedir(pDirectory);
	}
	
	return pItem;
}

Arxx::Item * pAddItem(Arxx::Archive & Archive, const Arxx::URI & ItemURI)
{
	if(g_bVerbose == true)
	{
		std::cerr << "Adding \"" << ItemURI << "\" ..." << std::endl;
	}
	
	Arxx::Item * pItem(0);
	
	if((ItemURI.sGetScheme() == "file") && (g_External == false))
	{
		struct stat Stat;
		
		if(stat(ItemURI.sGetPath().c_str(), &Stat) == -1)
		{
			switch(errno)
			{
			case EACCES:
				{
					throw std::runtime_error("The path \"" + ItemURI.sGetURI() + "\" contains non-browsable directories.");
				}
			case ELOOP:
				{
					throw std::runtime_error("The path \"" + ItemURI.sGetURI() + "\" contains a link loop.");
				}
			case ENOENT:
				{
					throw std::runtime_error("The file \"" + ItemURI.sGetURI() + "\" does not exist.");
				}
			default:
				{
					std::cerr << "Unknown \"stat()\" error: " << errno << std::endl;
				}
			}
		}
		if(S_ISREG(Stat.st_mode) == true)
		{
			pItem = pAddFile(Archive, ItemURI);
		}
		else if(S_ISDIR(Stat.st_mode) == true)
		{
			pItem = pAddDirectory(Archive, ItemURI);
		}
	}
	else
	{
		pItem = Archive.pGetItemFactory()->pNewItem(Archive);
		pItem->vSetExternal(ItemURI);
		vAssignUniqueID(pItem);
	}
	if(pItem != 0)
	{
		if(g_sName != "")
		{
			pItem->vSetName(g_sName);
			g_sName = "";
		}
		else
		{
			pItem->vSetName(ItemURI.sGetPath().substr(ItemURI.sGetPath().rfind('/') + 1));
		}
		pItem->vSetType(g_u4Type);
		pItem->vSetSubType(g_u4SubType);
		pItem->vSetVersion(g_u4Version);
		for(std::multiset< std::pair< Arxx::u4byte, std::string > >::iterator AddToRelationIterator = g_AddToRelations.begin(); AddToRelationIterator != g_AddToRelations.end(); ++AddToRelationIterator)
		{
			Arxx::Item * AddToItem(Archive.pGetItem(AddToRelationIterator->first));
			
			if(AddToItem != 0)
			{
				AddToItem->GetStructure().bAdd(pItem->u4GetUniqueID(), AddToRelationIterator->second);
			}
		}
		g_AddToRelations.clear();
	}
	
	return pItem;
}

int main(int argc, char **argv)
{
	Arxx::Repository.bRegisterDataChannel(&g_LocalFileChannel);
	
	std::vector< std::string > Parameters(argv + 1, argv + argc);
	
	if(bParameters(Parameters) == false)
	{
		return 1;
	}
	
	Arxx::URI ArchiveURI;
	Arxx::Archive Archive;
	bool bArchiveLoaded = false;
	
	for(std::vector< std::string >::iterator iParameter(Parameters.begin()); iParameter != Parameters.end(); ++iParameter)
	{
		if(*iParameter == "--uid")
		{
			g_u4UniqueID = GetHexValue(*(++iParameter));
		}
		else if(*iParameter == "--name")
		{
			g_sName = *(++iParameter);
		}
		else if(*iParameter == "--type")
		{
			g_u4Type = GetHexValue(*(++iParameter));
		}
		else if(*iParameter == "--subtype")
		{
			g_u4SubType = GetHexValue(*(++iParameter));
		}
		else if(*iParameter == "--version")
		{
			g_u4Version = GetHexValue(*(++iParameter));
		}
		else if(*iParameter == "--add")
		{
			++iParameter;
			if(bArchiveLoaded == true)
			{
				std::string::size_type Delimiter(iParameter->rfind("::"));
				std::string ItemIdentifier;
				std::string RelationName("child");
				
				if(Delimiter == std::string::npos)
				{
					ItemIdentifier = *iParameter;
				}
				else
				{
					ItemIdentifier = iParameter->substr(0, Delimiter);
					RelationName = iParameter->substr(Delimiter + 2);
				}
				
				Arxx::u4byte UniqueItemIdentifier(Arxx::g_u4InvalidID);
				
				if(IsHexValue(ItemIdentifier) == true)
				{
					UniqueItemIdentifier = GetHexValue(ItemIdentifier);
				}
				else
				{
					Arxx::Item * Item(Archive.GetItem(ItemIdentifier));
					
					if(Item != 0)
					{
						UniqueItemIdentifier = Item->u4GetUniqueID();
					}
					else
					{
						throw std::runtime_error("Could not find the item at path \"" + ItemIdentifier + "\".");
					}
				}
				g_AddToRelations.insert(std::make_pair(UniqueItemIdentifier, RelationName));
			}
			else
			{
				throw std::runtime_error("No archive has been loaded yet. A loaded archive is necessary to resolve --add calls.");
			}
		}
		else if(*iParameter == "--compress")
		{
			g_bCompress = true;
		}
		else if(*iParameter == "--external")
		{
			g_External = true;
		}
		else if(*iParameter == "--internal")
		{
			g_External = false;
		}
		else if(*iParameter == "--threshold")
		{
			std::istringstream ssStream(*(++iParameter));
			Arxx::u4byte u4Threshold;
			
			ssStream >> u4Threshold;
			g_u4CompressThreshold = u4Threshold;
		}
		else if(*iParameter == "--verbose")
		{
			g_bVerbose = !g_bVerbose;
		}
		else
		{
			if(bArchiveLoaded == false)
			{
				ArchiveURI = *iParameter;
				Archive.bLoad(ArchiveURI);
				bArchiveLoaded = true;
			}
			else
			{
				Arxx::URI URI(*iParameter);
				
				if(URI.sGetScheme() == "")
				{
					URI.vSetScheme("file");
				}
				try
				{
					pAddItem(Archive, URI);
				}
				catch(std::runtime_error & Exception)
				{
					std::cerr << Exception.what() << std::endl;
					
					return 1;
				}
			}
		}
	}
	Archive.vSave(ArchiveURI);
	
	return 0;
}
