/**
 * arxtools - Tools for modififying ARX archives.
 * Copyright (C) 2005  Hagen Möbius
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/

#include <iomanip>
#include <iostream>

#include "common.h"

int iItemInformation(Arxx::Item & Item, const Arxx::URI & URI);
int iRelationInformation(Arxx::Structure::Relation & Relation, const Arxx::URI & URI);

bool bParameters(const std::vector< std::string > & Parameters)
{
	if((Parameters.size() > 0) && (Parameters[0] == "--information"))
	{
		std::cerr << "arxget as part of " PACKAGE_STRING "\n\n";
		std::cerr << "No warranty, whatsoever. Copyright by Hagen Moebius, 2005\n" << std::endl;
		
		return false;
	}
	if((Parameters.size() > 0) && (Parameters[0] == "--version"))
	{
		std::cerr << PACKAGE_VERSION << std::endl;
		
		return false;
	}
	if((Parameters.size() == 0) || (Parameters[0] == "--help"))
	{
		std::cerr << "Usage: arxget <url>+\n\n";
		std::cerr << "Examples for url:\n";
		std::cerr << "  url = file:///some/path/archive.arx\n";
		std::cerr << "     -> gives information about an archive\n\n";
		std::cerr << "  url = file:///some/path/archive.arx?uid=1234\n";
		std::cerr << "     -> gives information about a specific item\n\n";
		std::cerr << "  url = file:///some/path/archive.arx?uid=1234&relation=child\n";
		std::cerr << "     -> gives information about the \"child\" relation of an item\n\n";
		std::cerr << "Append a fragment (i.e. \"#item-count\" for archives) to get specific information\n\n";
		std::cerr << "No warranty, whatsoever. Copyright by Hagen Moebius, 2005\n" << std::endl;
		
		return false;
	}
	
	return true;
}

int iArchiveInformation(Arxx::Archive & Archive, const Arxx::URI & URI)
{
	if(URI.sGetParameter("uid") != "")
	{
		Arxx::u4byte UniqueIdentifier(GetHexValue(URI.sGetParameter("uid")));
		Arxx::Item * pItem(Archive.pGetItem(UniqueIdentifier));
		
		if(pItem != 0)
		{
			int iResult(iItemInformation(*pItem, URI));
			
			if(iResult != 0)
			{
				return iResult;
			}
		}
		else
		{
			std::cerr << "An item with the unique identifier x" << std::hex << std::setw(8) << std::setfill('0') << std::right << UniqueIdentifier << " does not exist." << std::endl;
			
			return 1;
		}
	}
	else if(URI.sGetParameter("path") != "")
	{
		if(URI.sGetParameter("path")[0] != '/')
		{
			std::cerr << "If you specify items by path only, the path must be absolute (starting with '/')." << std::endl;
			
			return 1;
		}
		
		std::string sItemPath(URI.sGetParameter("path"));
		Arxx::Item * pItem(Archive.GetItem(sItemPath));
		
		if(pItem != 0)
		{
			int iResult(iItemInformation(*pItem, URI));
			
			if(iResult != 0)
			{
				return iResult;
			}
		}
	}
	else
	{
		if(URI.sGetFragment() == "")
		{
			std::cout << "item-count: " << Archive.size() << std::endl;
			std::cout << "item-uids:";
			for(Arxx::Archive::iterator iItem = Archive.begin(); iItem != Archive.end(); ++iItem)
			{
				std::cout << ' ' << std::hex << iItem->u4GetUniqueID();
			}
			std::cout << std::endl;
			std::cout << "rootitem-uid: ";
			if(Archive.pGetRootItem() == 0)
			{
				std::cout << "<none>";
			}
			else
			{
				std::cout << std::hex << Archive.pGetRootItem()->u4GetUniqueID();
			}
			std::cout << std::endl;
		}
		else if(URI.sGetFragment() == "item-count")
		{
			std::cout << Archive.size() << std::endl;
		}
		else if(URI.sGetFragment() == "item-uids")
		{
			for(Arxx::Archive::iterator iItem = Archive.begin(); iItem != Archive.end(); ++iItem)
			{
				std::cout << std::hex << iItem->u4GetUniqueID() << std::endl;
			}
		}
		else if(URI.sGetFragment() == "rootitem-uid")
		{
			if(Archive.pGetRootItem() != 0)
			{
				std::cout << std::hex << Archive.pGetRootItem()->u4GetUniqueID() << std::endl;
			}
		}
		else
		{
			std::cerr << "Unrecognized fragment \"" << URI.sGetFragment() << "\" on archives." << std::endl;
			
			return 1;
		}
	}
	
	return 0;
}

int iItemInformation(Arxx::Item & Item, const Arxx::URI & URI)
{
	if(URI.sGetParameter("relation") != "")
	{
		if(Item.GetStructure().bHasRelation(URI.sGetParameter("relation")) == false)
		{
			std::cerr << "The item \"" << Item.sGetName() << "\"[x" << std::hex << std::setw(8) << std::setfill('0') << std::right << Item.u4GetUniqueID() << "] does not have a relation named \"" << URI.sGetParameter("relation") << "\"." << std::endl;
			
			return 1;
		}
		else
		{
			Arxx::Structure::Relation & Relation(Item.GetStructure().GetRelation(URI.sGetParameter("relation")));
			
			return iRelationInformation(Relation, URI);
		}
	}
	else
	{
		if(URI.sGetFragment() == "")
		{
			std::cout << "uid: " << std::hex << Item.u4GetUniqueID() << std::endl;
			std::cout << "name: " << Item.sGetName() << std::endl;
			std::cout << "type: " << std::hex << Item.u4GetType() << std::endl;
			std::cout << "subtype: " << std::hex << Item.u4GetSubType() << std::endl;
			std::cout << "version: " << Item.u4GetVersion() << std::endl;
			std::cout << "data-compression: " << std::dec << Item.GetCompression() << std::endl;
			std::cout << "data-compressedlength: " << Item.u4GetCompressedLength() << std::endl;
			std::cout << "data-decompressedlength: " << Item.u4GetDecompressedLength() << std::endl;
			std::cout << "data-external: " << Item.bIsExternal() << std::endl;
			if(Item.bIsExternal() == true)
			{
				std::cout << "data-uri: " << Item.GetURI() << std::endl;
			}
			else
			{
				std::cout << "data-uri: " << std::endl;
			}
			if(Item.bIsFetched() == false)
			{
				Item.bFetch();
			}
			if(Item.bIsFetched() == true)
			{
				if(Item.GetCompression() > 0)
				{
					Item.vDecompress();
				}
				if(Item.GetCompression() == 0)
				{
					std::cout << "data: " << Item << std::endl;
				}
				else
				{
					std::cerr << "data: <decompress failed>" << std::endl;
				}
			}
			else
			{
				std::cout << "data: <fetch failed>" << std::endl;
			}
			std::cout << "relation-count: " << Item.GetStructure().size() << std::endl;
			std::cout << "relation-names:";
			for(Arxx::Structure::iterator iRelation = Item.GetStructure().begin(); iRelation != Item.GetStructure().end(); ++iRelation)
			{
				std::cout << ' ' << iRelation->sGetName();
			}
			std::cout << std::endl;
		}
		else if(URI.sGetFragment() == "uid")
		{
			std::cout << std::hex << Item.u4GetUniqueID() << std::endl;
		}
		else if(URI.sGetFragment() == "name")
		{
			std::cout << Item.sGetName() << std::endl;
		}
		else if(URI.sGetFragment() == "type")
		{
			std::cout << std::hex << Item.u4GetType() << std::endl;
		}
		else if(URI.sGetFragment() == "subtype")
		{
			std::cout << std::hex << Item.u4GetSubType() << std::endl;
		}
		else if(URI.sGetFragment() == "version")
		{
			std::cout << Item.u4GetVersion() << std::endl;
		}
		else if(URI.sGetFragment() == "data-compression")
		{
			std::cout << std::dec << Item.GetCompression() << std::endl;
		}
		else if(URI.sGetFragment() == "data-compressedlength")
		{
			std::cout << Item.u4GetCompressedLength() << std::endl;
		}
		else if(URI.sGetFragment() == "data-decompressedlength")
		{
			std::cout << Item.u4GetDecompressedLength() << std::endl;
		}
		else if(URI.sGetFragment() == "data-external")
		{
			std::cout << Item.bIsExternal() << std::endl;
		}
		else if(URI.sGetFragment() == "data-uri")
		{
			if(Item.bIsExternal() == true)
			{
				std::cout << Item.GetURI() << std::endl;
			}
			else
			{
				std::cout << std::endl;
			}
		}
		else if(URI.sGetFragment() == "data")
		{
			if(Item.bIsFetched() == false)
			{
				Item.bFetch();
			}
			if(Item.bIsFetched() == true)
			{
				if(Item.GetCompression() > 0)
				{
					Item.vDecompress();
				}
				if(Item.GetCompression() == 0)
				{
					std::cout << Item;
				}
				else
				{
					std::cerr << "Decompression the data was unsuccessful." << std::endl;
					
					return 1;
				}
			}
			else
			{
				std::cerr << "Fetching the data was unsuccessful." << std::endl;
				
				return 1;
			}
		}
		else if(URI.sGetFragment() == "relation-count")
		{
			std::cout << Item.GetStructure().size() << std::endl;
		}
		else if(URI.sGetFragment() == "relation-names")
		{
			for(Arxx::Structure::iterator iRelation = Item.GetStructure().begin(); iRelation != Item.GetStructure().end(); ++iRelation)
			{
				std::cout << iRelation->sGetName() << std::endl;
			}
		}
		else
		{
			std::cerr << "Unrecognized fragment \"" << URI.sGetFragment() << "\" on items." << std::endl;
			
			return 1;
		}
	}
	
	return 0;
}

int iRelationInformation(Arxx::Structure::Relation & Relation, const Arxx::URI & URI)
{
	if(URI.sGetFragment() == "")
	{
		std::cout << "name: " << Relation.sGetName() << std::endl;
		std::cout << "reference-count: " << Relation.size() << std::endl;
		std::cout << "reference-uids:";
		for(Arxx::Structure::Relation::const_iterator iReference = Relation.begin(); iReference != Relation.end(); ++iReference)
		{
			std::cout << ' ' << std::hex << iReference->u4GetUniqueID();
		}
		std::cout << std::endl;
	}
	else if(URI.sGetFragment() == "name")
	{
		std::cout << Relation.sGetName() << std::endl;
	}
	else if(URI.sGetFragment() == "reference-count")
	{
		std::cout << Relation.size() << std::endl;
	}
	else if(URI.sGetFragment() == "reference-uids")
	{
		for(Arxx::Structure::Relation::const_iterator iReference = Relation.begin(); iReference != Relation.end(); ++iReference)
		{
			std::cout << std::hex << iReference->u4GetUniqueID() << std::endl;
		}
	}
	else
	{
		std::cerr << "Unrecognized fragment \"" << URI.sGetFragment() << "\" on relations." << std::endl;
		
		return 1;
	}
	
	return 0;
}

int main(int argc, char ** argv)
{
	std::vector< std::string > Parameters(argv + 1, argv + argc);
	
	if(bParameters(Parameters) == false)
	{
		return 1;
	}
	
	for(std::vector< std::string >::iterator iParameter = Parameters.begin(); iParameter != Parameters.end(); ++iParameter)
	{
		Arxx::URI URI(*iParameter);
		
		if(URI.sGetScheme() == "")
		{
			URI.vSetScheme("file");
		}
		if((URI.sGetScheme() == "file") && (URI.sGetPath()[0] != '/'))
		{
			URI.vSetPath(sGetCurrentWorkingDirectory() + '/' + URI.sGetPath());
		}
		
		Arxx::URI ArchiveURI(URI);
		
		ArchiveURI.vSetQuery("");
		ArchiveURI.vSetFragment("");
		
		Arxx::Archive Archive;
		
		if(Archive.bLoad(ArchiveURI) == false)
		{
			std::cerr << "Loading of archive \"" << ArchiveURI << "\" failed." << std::endl;
			
			return 1;
		}
		
		int iResult(iArchiveInformation(Archive, URI));
		
		if(iResult != 0)
		{
			// stop at the first error
			// is that sensible??
			return iResult;
		}
	}
	
	return 0;
}
