/**
 * arxtools - Tools for modififying ARX archives.
 * Copyright (C) 2005  Hagen Möbius
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/

#include <iomanip>
#include <iostream>
#include <sstream>

#include "common.h"

void vHeader(void)
{
	std::cerr << std::setfill(' ');
	std::cerr << std::setw(8) << std::right << "UID" << "  ";
	std::cerr << std::setw(8) << std::right << "Type" << "  ";
	std::cerr << std::setw(8) << std::right << "Subtype" << "  ";
	std::cerr << std::setw(10) << std::right << "Compressed" << "  ";
	std::cerr << std::setw(11) << std::right << "Version" << "  ";
	std::cerr << std::setw(11) << std::right << "Length" << "        ";
	std::cerr << "Name" << std::endl;
}

void vListItem(const Arxx::Item & Item)
{
	std::cout << std::setw(8) << std::right << std::hex << Item.u4GetUniqueID() << "  ";
	std::cout << std::setw(8) << std::right << std::hex << std::setfill('0') << Item.u4GetType() << "  ";
	std::cout << std::setw(8) << std::right << std::hex << std::setfill('0') << Item.u4GetSubType() << "  ";
	std::cout << std::setw(10) << std::right << std::boolalpha << std::setfill(' ') << Item.bIsCompressed() << "  ";
	std::cout << std::setw(2) << std::right << std::hex << std::setfill('0') << ((Item.u4GetVersion() & 0xFF000000) >> 24) << '.';
	std::cout << std::setw(2) << std::right << std::hex << std::setfill('0') << ((Item.u4GetVersion() & 0x00FF0000) >> 16) << '.';
	std::cout << std::setw(2) << std::right << std::hex << std::setfill('0') << ((Item.u4GetVersion() & 0x0000FF00) >> 8) << '.';
	std::cout << std::setw(2) << std::right << std::hex << std::setfill('0') << (Item.u4GetVersion() & 0x000000FF) << "  ";
	std::cout << std::setw(8) << std::right << std::dec << std::setfill(' ') << Item.u4GetDecompressedLength() << "/" << std::setw(8) << std::left << Item.u4GetCompressedLength() << "  ";
	std::cout << Item.sGetName();
	if(Item.bIsExternal() == true)
	{
		std::cout << "  =>  " << Item.GetURI();
	}
	std::cout << std::endl;
}

void vListItemChilds(const Arxx::Item & Item)
{
	Arxx::Structure::const_iterator iRelation(Item.GetStructure().begin());
	Arxx::Structure::const_iterator iEnd(Item.GetStructure().end());
	
	while(iRelation != iEnd)
	{
		std::cout << "Relation = \"" << iRelation->sGetName() << '"' << std::endl;
		vHeader();
		
		Arxx::Structure::Relation::const_iterator iItem(iRelation->begin());
		
		while(iItem != iRelation->end())
		{
			if(iItem->pGetItem() == 0)
			{
				std::cout << std::setw(8) << std::right << std::hex << iItem->u4GetUniqueID() << "  ";
				std::cout << "<no item with this id>";
				std::cout << std::endl;
			}
			else
			{
				vListItem(*(iItem->pGetItem()));
			}
			++iItem;
		}
		++iRelation;
	}
}

const Arxx::Item * pGetItemAtPath(const Arxx::Item * pRoot, const std::string & _sPath, const std::string & sChildRelation = "child")
{
	if(pRoot == 0)
	{
		return 0;
	}
	
	std::string sPath(_sPath);
	
	if(sPath[0] == '/')
	{
		sPath = sPath.substr(1);
	}
	
	std::string::size_type stDelimiter(sPath.find('/'));
	
	if(stDelimiter == std::string::npos)
	{
		stDelimiter = sPath.length();
	}
	
	std::string sChildName(sPath.substr(0, stDelimiter));
	
	if(sChildName == "")
	{
		return pRoot;
	}
	
	if(pRoot->GetStructure().bHasRelation(sChildRelation) == false)
	{
		/** @todo What to do here? **/
	}
	else
	{
		const Arxx::Structure::Relation & Relation(pRoot->GetStructure().GetRelation(sChildRelation));
		Arxx::Structure::Relation::const_iterator iItem(Relation.begin());
		
		while(iItem != Relation.end())
		{
			if((iItem->pGetItem() != 0) && (iItem->pGetItem()->sGetName() == sChildName))
			{
				return pGetItemAtPath(iItem->pGetItem(), sPath.substr(stDelimiter), sChildRelation);
			}
			++iItem;
		}
	}
	
	return 0;
}

bool bParameters(const std::vector< std::string > & Parameters)
{
	if((Parameters.size() > 0) && (Parameters[0] == "--information"))
	{
		std::cerr << "arxlist as part of " PACKAGE_STRING "\n\n";
		std::cerr << "No warranty, whatsoever. Copyright by Hagen Moebius, 2005\n" << std::endl;
		
		return false;
	}
	if((Parameters.size() > 0) && (Parameters[0] == "--version"))
	{
		std::cerr << PACKAGE_VERSION << std::endl;
		
		return false;
	}
	if((Parameters.size() == 0) || (Parameters[0] == "--help"))
	{
		std::cerr << "Usage: arxlist <url>+\n\n";
		std::cerr << "Examples for url:\n";
		std::cerr << "  url = file:///some/path/archive.arx\n";
		std::cerr << "     -> lists all items inside the archive\n\n";
		std::cerr << "  url = file:///some/path/archive.arx?uid=1234\n";
		std::cerr << "     -> lists the item specified by uid\n";
		std::cerr << "     -> the uid must be given as hexadecimal value\n";
		std::cerr << "  url = file:///some/path/archive.arx?path=/\n";
		std::cerr << "     -> lists the root item of the archive\n\n";
		std::cerr << "No warranty, whatsoever. Copyright by Hagen Moebius, 2005\n" << std::endl;
		
		return false;
	}
	
	return true;
}

int main(int argc, char ** argv)
{
	std::vector< std::string > Parameters(argv + 1, argv + argc);
	
	if(bParameters(Parameters) == false)
	{
		return 1;
	}
	for(std::vector< std::string >::iterator iParameter = Parameters.begin(); iParameter != Parameters.end(); ++iParameter)
	{
		Arxx::URI URI(*iParameter);
		
		if(URI.sGetScheme() == "")
		{
			URI.vSetScheme("file");
		}
		if((URI.sGetScheme() == "file") && (URI.sGetPath()[0] != '/'))
		{
			URI.vSetPath(sGetCurrentWorkingDirectory() + '/' + URI.sGetPath());
		}
		
		Arxx::URI ArchiveURI(URI);
		
		ArchiveURI.vSetQuery("");
		ArchiveURI.vSetFragment("");
		
		Arxx::Archive Archive;
		
		if(Archive.bLoad(ArchiveURI) == false)
		{
			std::cerr << "Loading of archive \"" << ArchiveURI << "\" failed." << std::endl;
			
			return 1;
		}
		if(URI.sGetParameter("path") != "")
		{
			const Arxx::Item * pItem(pGetItemAtPath(Archive.pGetRootItem(), URI.sGetParameter("path")));
			
			if(pItem == 0)
			{
				std::cerr << "No item at path \"" << URI.sGetParameter("path") << "\" found in archive \"" << ArchiveURI.sGetPath() << "\"." << std::endl;
			}
			else
			{
				vListItemChilds(*pItem);
			}
		}
		else if(URI.sGetParameter("uid") != "")
		{
			Arxx::u4byte UniqueIdentifier(GetHexValue(URI.sGetParameter("uid")));
			Arxx::Item * pItem(Archive.pGetItem(UniqueIdentifier));
			
			if(pItem == 0)
			{
				std::cerr << "No item with id \"" << std::hex << UniqueIdentifier << "\" found in the archive \"" << ArchiveURI.sGetPath() << "\"." << std::endl;
				
				return 1;
			}
			vListItemChilds(*pItem);
		}
		else
		{
			Arxx::Archive::iterator iBegin(Archive.begin());
			Arxx::Archive::iterator iEnd(Archive.end());
			
			if(iBegin != iEnd)
			{
				vHeader();
			}
			while(iBegin != iEnd)
			{
				vListItem(*iBegin);
				++iBegin;
			}
		}
	}
	
	return 0;
}
