/**
 * arxtools - Tools for modififying ARX archives.
 * Copyright (C) 2005  Hagen Möbius
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/

#include <sys/stat.h>

#include <iostream>

#include <Merge.h>

#include "common.h"

class Rule : public Arxx::Rule
{
public:
	Rule(const std::string & sName, Arxx::Action Action) :
		Arxx::Rule(Action),
		m_sName(sName)
	{
	}
	
	const std::string & sGetName(void) const
	{
		return m_sName;
	}
	
	virtual ~Rule(void)
	{
	}
private:
	std::string m_sName;
};

class drop_item_on_duplicate_id : public Rule
{
public:
	drop_item_on_duplicate_id(void) :
		Rule("drop item on duplicate id", Arxx::DROP)
	{
	}
	
	virtual bool bMatch(Arxx::Context & Context) const
	{
		return Context.GetTargetArchive().pGetItem(Context.GetCurrentItem().u4GetUniqueID()) != 0;
	}
} Drop;

class change_id_on_duplicate_id : public Rule
{
public:
	change_id_on_duplicate_id(void) :
		Rule("change id on duplicate id", Arxx::PASS)
	{
	}
	
	virtual bool bMatch(Arxx::Context & Context) const
	{
		return Context.GetTargetArchive().pGetItem(Context.GetCurrentItem().u4GetUniqueID()) != 0;
	}
	
	virtual void vAct(Arxx::Context & Context) const
	{
		Context.GetCurrentItem().vSetUniqueID(Arxx::g_u4InvalidID);
	}
} ChangeID;

class add_all : public Rule
{
public:
	add_all(void) :
		Rule("add all", Arxx::ADD)
	{
	}
	
	virtual bool bMatch(Arxx::Context & Context) const
	{
		return true;
	}
} Add;

class replace_on_newer_version : public Rule
{
public:
	replace_on_newer_version(void) :
		Rule("replace on newer version", Arxx::REPLACE)
	{
	}
	
	virtual bool bMatch(Arxx::Context & Context) const
	{
		const Arxx::Item * pItem(Context.GetTargetArchive().pGetItem(Context.GetCurrentItem().u4GetUniqueID()));
		
		if(pItem == 0)
		{
			return false;
		}
		else
		{
			return Context.GetCurrentItem().u4GetVersion() > pItem->u4GetVersion();
		}
	}
} ReplaceOnNewerVersion;

bool bParameters(std::vector< std::string > & Parameters, std::map< std::string, Rule * > & Filters)
{
	if((Parameters.size() > 0) && (Parameters[0] == "--information"))
	{
		std::cerr << "arxmerge as part of " PACKAGE_STRING "\n\n";
		std::cerr << "No warranty, whatsoever. Copyright by Hagen Moebius, 2005\n" << std::endl;
		
		return false;
	}
	if((Parameters.size() > 0) && (Parameters[0] == "--version"))
	{
		std::cerr << PACKAGE_VERSION << std::endl;
		
		return false;
	}
	if((Parameters.size() == 0) || (Parameters[0] == "--help") || (Parameters.size() < 3))
	{
		std::cerr << "Usage: arxmerge <arx archive> <arx archive> [<arx archive>] filters ...\n";
		std::cerr << "Define an arbitrary number of filters from the following:\n";
		for(std::map< std::string, Rule * >::iterator iFilter(Filters.begin()); iFilter != Filters.end(); ++iFilter)
		{
			std::cerr << '\t' << iFilter->first << "  --  " << iFilter->second->sGetName() << '\n';
		}
		std::cerr << "Note that the order of filters is important.\n\n";
		std::cerr << "No warranty, whatsoever. Copyright by Hagen Moebius, 2005\n" << std::endl;
		
		return false;
	}
	
	return true;
}

int main(int argc, char **argv)
{
	std::map< std::string, Rule * > Filters;
	
	Filters.insert(std::make_pair("-ddid", &Drop));
	Filters.insert(std::make_pair("-cdid", &ChangeID));
	Filters.insert(std::make_pair("-a", &Add));
	Filters.insert(std::make_pair("-rnv", &ReplaceOnNewerVersion));
	
	std::vector< std::string > Parameters(argv + 1, argv + argc);
	std::vector< Rule * > Rules;
	std::vector< std::string >::iterator iParameter(Parameters.begin());
	
	while(iParameter != Parameters.end())
	{
		std::map< std::string, Rule * >::iterator iRule(Filters.find(*iParameter));
		
		if(iRule != Filters.end())
		{
			Rules.push_back(iRule->second);
			Parameters.erase(iParameter);
		}
		else
		{
			++iParameter;
		}
	}
	if(bParameters(Parameters, Filters) == false)
	{
		return 1;
	}
	std::cout << "Ruleset is:\n";
	for(std::vector< Rule * >::size_type stRule = 0; stRule < Rules.size(); ++stRule)
	{
		std::cout << '\t' << stRule << ". - " << Rules[stRule]->sGetName() << '\n';
	}
	std::cout << std::flush;
	
	Arxx::Archive Archive1;
	
	if(Archive1.bLoad(Parameters[0]) == false)
	{
		std::cerr << "Loading the archive \"" << Parameters[0] << "\" failed." << std::endl;
		
		return 1;
	}
	
	Arxx::Archive Archive2;
	
	if(Archive2.bLoad(Parameters[1]) == false)
	{
		std::cerr << "Loading the archive \"" << Parameters[1] << "\" failed." << std::endl;
		
		return 1;
	}
	
	std::vector< Arxx::Rule * > MergeRules(Rules.begin(), Rules.end());
	
	if(Archive1.bMerge(Archive2, MergeRules) == false)
	{
		std::cerr << "Merging the archives \"" << Parameters[0] << "\" and \"" << Parameters[1] << "\" failed." << std::endl;
		
		return 1;
	}
	if(Parameters.size() > 2)
	{
		Archive1.vSave(Parameters[2]);
	}
	else
	{
		Archive1.vSave(Parameters[0]);
	}
}
