/**
 * arxtools - Tools for modififying ARX archives.
 * Copyright (C) 2005  Hagen Möbius
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/

#include "common.h"

#include <iostream>

bool bParameters(std::vector< std::string > &Parameters)
{
	if((Parameters.size() > 0) && (Parameters[0] == "--information"))
	{
		std::cerr << "arxmod as part of " PACKAGE_STRING "\n\n";
		std::cerr << "No warranty, whatsoever. Copyright by Hagen Moebius, 2005\n" << std::endl;
		
		return false;
	}
	if((Parameters.size() > 0) && (Parameters[0] == "--version"))
	{
		std::cerr << PACKAGE_VERSION << std::endl;
		
		return false;
	}
	if((Parameters.size() == 0) || (Parameters[0] == "--help") || (Parameters.size() < 3) || (((Parameters[2] != "-c") || (Parameters[2] != "-drop")) && (Parameters.size() < 2)))
	{
		std::cerr << "Usage: arxmod <arx archive> (<uid> (<option> [parameter1 [parameter2]])+)+\n\n";
		std::cerr << "<option> may be one of the following:\n";
		std::cerr << "\t-id   -  change the id of the item, parameter1 is a hex-value, no parameter 2\n";
		std::cerr << "\t-t    -  change the type of the item, parameter1 is a hex-value, no parameter 2\n";
		std::cerr << "\t-st   -  change the subtype of the item, parameter1 is a hex-value, no parameter 2\n";
		std::cerr << "\t-n    -  change the name of the item, parameter1 is a string-value, no parameter 2\n";
		std::cerr << "\t-v    -  change the version of the item, parameter1 is a hex-value, no parameter 2\n";
		std::cerr << "\t-a    -  adds to the relation in parameter1 (name) the item in parameter2 (uid as hex-value)\n";
		std::cerr << "\t-r    -  removes from the relation in parameter1 (name) the item in parameter2 (uid as hex-value)\n";
		std::cerr << "\t-c    -  compresse the item, no parameter\n";
		std::cerr << "\t-d    -  decompresse the item, no parameter\n";
		std::cerr << "\t-drop -  drop the item off the archive\n";
		std::cerr << "\t-root -  make the item the archive's root item\n\n";
		std::cerr << "No warranty, whatsoever. Copyright by Hagen Moebius, 2005\n";
		std::cerr << std::endl;
		
		return false;
	}
	
	return true;
}

int main(int argc, char **argv)
{
	std::vector< std::string > Parameters(argv + 1, argv + argc);
	
	if(bParameters(Parameters) == false)
	{
		return 1;
	}
	
	Arxx::Archive Archive;
	
	if(Archive.bLoad(Parameters[0]) == false)
	{
		std::cerr << "Loading the archive \"" << Parameters[0] << "\" failed." << std::endl;
		
		return 1;
	}
	
	std::vector< std::string >::iterator iParameter(Parameters.begin() + 1);
	Arxx::Item * pItem(0);
	
	while(iParameter != Parameters.end())
	{
		try
		{
			if((*iParameter)[0] != '-')
			{
				pItem = Archive.pGetItem(GetHexValue(*iParameter));
				if(pItem == 0)
				{
					std::cerr << "Item with unique ID \"" << *iParameter << "\" not found." << std::endl;
					
					return 1;
				}
			}
			else
			{
				if(pItem == 0)
				{
					std::cerr << "No item has been selected for operation \"" << *iParameter << "\"." << std::endl;
					
					return 1;
				}
				if(*iParameter == "-id")
				{
					pItem->vSetUniqueID(GetHexValue(*(++iParameter)));
				}
				else if(*iParameter == "-t")
				{
					pItem->vSetType(GetHexValue(*(++iParameter)));
				}
				else if(*iParameter == "-st")
				{
					pItem->vSetSubType(GetHexValue(*(++iParameter)));
				}
				else if(*iParameter == "-n")
				{
					pItem->vSetName(*(++iParameter));
				}
				else if(*iParameter == "-v")
				{
					pItem->vSetVersion(GetHexValue(*(++iParameter)));
				}
				else if(*iParameter == "-a")
				{
					std::string sRelation(*(++iParameter));
					
					pItem->GetStructure().bAdd(GetHexValue(*(++iParameter)), sRelation);
				}
				else if(*iParameter == "-r")
				{
					std::string sRelation(*(++iParameter));
					
					pItem->GetStructure().bDelete(GetHexValue(*(++iParameter)), sRelation);
				}
				else if(*iParameter == "-c")
				{
					pItem->bFetch();
					pItem->vCompress();
				}
				else if(*iParameter == "-d")
				{
					pItem->bFetch();
					pItem->vDecompress();
				}
				else if(*iParameter == "-drop")
				{
					Archive.vUnregisterItem(*pItem);
				}
				else if(*iParameter == "-root")
				{
					Archive.vSetRootItem(pItem);
				}
			}
		}
		catch(Arxx::id_not_unique & Exception)
		{
			std::cerr << "The new ID \"" << Exception.what() << "\" is not unique." << std::endl;
		}
		catch(std::exception & Exception)
		{
			std::cerr << Exception.what() << std::endl;
		}
		++iParameter;
	}
	Archive.vSave(argv[1]);
	
	return 0;
}
