/**
 * arxtools - Tools for modififying ARX archives.
 * Copyright (C) 2005  Hagen Möbius
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/

#include <errno.h>
#include <sys/stat.h>

#include <iostream>

#include "common.h"

bool bIsDecValue(const std::string & sString);

bool bIsNumericValue(const std::string & sString)
{
	if(sString[0] == 'x')
	{
		return IsHexValue(sString.substr(1));
	}
	else
	{
		return bIsDecValue(sString) || IsHexValue(sString);
	}
}

bool bIsDecValue(const std::string & sDecString)
{
	char cDigit('\0');
	
	for(std::string::size_type stI = 0; stI < sDecString.length(); ++stI)
	{
		cDigit = sDecString[stI];
		if((cDigit < '0') || (cDigit > '9'))
		{
			return false;
		}
	}
	
	return true;
}

bool IsHexValue(const std::string & sHex)
{
	std::string::size_type stI(0);
	char cHex('\0');
	
	while(stI < sHex.length())
	{
		cHex = sHex[stI];
		if(((cHex < 'a') || (cHex > 'f')) && ((cHex < 'A') || (cHex > 'F')) && ((cHex < '0') || (cHex > '9')))
		{
			return false;
		}
		++stI;
	}
	
	return true;
}

Arxx::u4byte GetHexValue(const std::string & sHex)
{
	std::string::size_type stI = 0;
	Arxx::u4byte u4Return = 0;
	char cHex = 0;
	
	while(stI < sHex.length())
	{
		u4Return <<= 4;
		cHex = sHex[stI];
		if((cHex >= 'a') && (cHex <= 'f'))
		{
			u4Return += cHex - 'a' + 10;
		}
		else
		{
			if((cHex >= 'A') && (cHex <= 'F'))
			{
				u4Return += cHex - 'A' + 10;
			}
			else
			{
				if((cHex >= '0') && (cHex <= '9'))
				{
					u4Return += cHex - '0';
				}
			}
		}
		stI++;
	}
	
	return u4Return;
}

Arxx::u4byte u4GetDecValue(std::string sDecString)
{
	Arxx::u4byte u4Result(0);
	char cDigit(0);
	
	for(std::string::size_type stI = 0; stI < sDecString.length(); ++stI)
	{
		u4Result *= 10;
		cDigit = sDecString[stI];
		if((cDigit >= '0') || (cDigit <= '9'))
		{
			u4Result += cDigit - '0';
		}
	}
	
	return u4Result;
}

bool bFileExists(const std::string & sFilePath)
{
	struct stat Stat;
	
	if(stat(sFilePath.c_str(), &Stat) == -1)
	{
		switch(errno)
		{
		case EACCES:
			{
				std::cerr << "Access denied to file: \"" << sFilePath << "\"." << std::endl;
				
				break;
			}
		case ELOOP:
			{
				std::cerr << "Symbolic link loop for file: \"" << sFilePath << "\"." << std::endl;
				
				break;
			}
		}
		
		return false;
	}
	
	return true;
}

std::string sGetCurrentWorkingDirectory(void)
{
	static size_t stBufferSize(128);
	static char * pcBuffer(new char[stBufferSize]);
	
	while(getcwd(pcBuffer, stBufferSize) == 0)
	{
		if(errno != ERANGE)
		{
			throw std::runtime_error("getcwd returned 0 and errno was not equal to ERANGE");
		}
		delete [] pcBuffer;
		stBufferSize <<= 1;
		pcBuffer = new char[stBufferSize];
	}
	
	return pcBuffer;
}

Arxx::u4byte u4GetValue(const std::string & sValueString)
{
	if(sValueString[0] == 'x')
	{
		if(IsHexValue(sValueString.substr(1)) == true)
		{
			return GetHexValue(sValueString.substr(1));
		}
	}
	else
	{
		if(bIsDecValue(sValueString) == true)
		{
			return u4GetDecValue(sValueString);
		}
		else if(IsHexValue(sValueString) == true)
		{
			return GetHexValue(sValueString);
		}
	}
}
