#!/bin/bash

PATH=../Source/:$PATH

GREEN=`echo -en "\033[32m"`
RED=`echo -en "\033[31;1m"`
WHITE=`echo -en "\033[0m"`
YELLOW=`echo -en "\033[33;1m"`
I=0

setup_test()
{
	(( ++I ))
	echo -e -n ${YELLOW}$I. Test:${WHITE} $1 ...
}

run_test()
{
	output=`$@ 2> test$I.err`
	result=$?
}

check_test()
{
	if [ "x$1" = "x" ]; then
		expect_output=""
	else
		expect_output=$1
	fi
	if [ "x$2" = "x" ]; then
		expect_result=0
	else
		expect_result=$2
	fi
	if [ $result -ne $expect_result -o "$output" != "$expect_output" ]; then
		echo ${RED}" failed"${WHITE}.
		echo
		if [ $result -ne $expect_result ]; then
			echo -e "\tExpected result: $expect_result"
			echo -e "\t         result: $result"
			echo
			if [ -f "test$I.err" ]; then
				echo -e "\t   Error output: "
				cat "test$I.err"
			fi
		fi
		if [ "$output" != "$expect_output" ]; then
			echo -e "\tExpected output: $expect_output"
			echo -e "\t         output: $output"
			echo
		fi
	else
		echo ${GREEN}" passed"${WHITE}.
	fi
	rm -f "test$I.err"
}

rm -f Test.arx

setup_test "testing arxtouch with a library"
run_test arxtouch Test.arx
check_test

setup_test "testing zero items in new library"
run_test arxget "Test.arx#item-count"
check_test "0"

setup_test "testing arxtouch with a library and an item"
run_test arxtouch Test.arx 1
check_test

setup_test "testing one item in new library"
run_test arxget "Test.arx#item-count"
check_test "1"

setup_test "testing arxtouch with a library and multiple item"
run_test arxtouch Test.arx 2 3 4
check_test

setup_test "testing four items in new library"
run_test arxget "Test.arx#item-count"
check_test "4"

setup_test "testing arxtouch with some invalid items"
run_test arxtouch Test.arx 2 3 5 1 6
check_test

setup_test "testing six items in new library"
run_test arxget "Test.arx#item-count"
check_test "6"

setup_test "testing the six item uids in new library"
run_test arxget "Test.arx#item-uids"
check_test "`echo -e "1\n2\n3\n4\n5\n6"`"

setup_test "setting name to the item 1"
run_test arxmod Test.arx 1 -n "Item1"
check_test

setup_test "checking name of item 1"
run_test arxget "Test.arx?uid=1#name"
check_test "Item1"

setup_test "setting name to the item 2"
run_test arxmod Test.arx 2 -n "Item2"
check_test

setup_test "setting name to items 3 and 4"
run_test arxmod Test.arx 3 -n "Item3" 4 -n "Item4"
check_test

setup_test "checking name of item 3"
run_test arxget "Test.arx?uid=3#name"
check_test "Item3"

setup_test "checking name of item 4"
run_test arxget "Test.arx?uid=4#name"
check_test "Item4"

setup_test "check names for item 1, 2, 3 and 4"
run_test arxget "Test.arx?uid=1#name" "Test.arx?uid=2#name" "Test.arx?uid=3#name" "Test.arx?uid=4#name"
check_test "`echo -e "Item1\nItem2\nItem3\nItem4"`"

setup_test "add item 2 reference to item 1, relation \"first\""
run_test arxmod Test.arx 1 -a first 2
check_test

setup_test "check relation \"first\" presence on item 1"
run_test arxget "Test.arx?uid=1&relation=first#name"
check_test "first"

setup_test "check relation \"second\" absence on item 1"
run_test arxget "Test.arx?uid=1&relation=second#name"
check_test "" 1

setup_test "add item 3, 4 and 5 to item 1, relation \"second\""
run_test arxmod Test.arx 1 -a second 3 -a second 4 -a second 5
check_test

setup_test "check relation \"second\" presence and 3 references on item 1"
run_test arxget "Test.arx?uid=1&relation=second#name" "Test.arx?uid=1&relation=second#reference-count"
check_test "`echo -e "second\n3"`"

setup_test "check the three references in relation \"second\" on item 1"
run_test arxget "Test.arx?uid=1&relation=second#reference-uids"
check_test "`echo -e "3\n4\n5"`"

setup_test "testing arxtouch with more hexadecimal values"
run_test arxtouch Test.arx 30 31 4f a2
check_test

setup_test "testing item uids in library"
run_test arxget "Test.arx#item-uids"
check_test "`echo -e "1\n2\n3\n4\n5\n6\n30\n31\n4f\na2"`"

setup_test "setting name with implicite hex value 4f"
run_test arxmod Test.arx 4f -n Item79d
check_test

setup_test "testing name on item 79"
run_test arxget "Test.arx?uid=4f#name"
check_test "Item79d"

setup_test "setting item a2 as root item"
run_test arxmod Test.arx a2 -root
check_test

setup_test "testing root item a2"
run_test arxget "Test.arx#rootitem-uid"
check_test "a2"

setup_test "add item 1 to item a2 [\"child\"]"
run_test arxmod Test.arx a2 -a child 1
check_test

setup_test "add item 5 to item 1 [\"child\"]"
run_test arxmod Test.arx 1 -a child 5
check_test

setup_test "add file Makefile as item 10 and add to 2::add_test"
run_test arxadd Test.arx --uid 10 --add 2::add_test Makefile
check_test

setup_test "testing whether 10 is in 2::add_test"
run_test arxget "Test.arx?uid=2&relation=add_test#reference-uids"
check_test "10"

setup_test "add file Makefile as item 11 and add to /::add_test"
run_test arxadd Test.arx --uid 11 --add /::add_test Makefile
check_test

setup_test "testing whether 11 is in /::add_test"
run_test arxget "Test.arx?path=/&relation=add_test#reference-uids"
check_test "11"

setup_test "add file Makefile as item MAKEFILE and add to /Item1/second::Item4/::add_test"
run_test arxadd Test.arx --name MAKEFILE --uid 12 --add /Item1/second::Item4/::add_test Makefile
check_test

setup_test "testing whether 12 is /Item1/second::Item4/add_test::MAKEFILE"
run_test arxget "Test.arx?path=/Item1/second::Item4/add_test::MAKEFILE#uid"
check_test "12"
